import json
from request import ask
from functionstime import get_time_zone
from util import _convert_specific_format
from util import ExchangeMonitor

def get_exchange_rate(types: list, country: str = 've'):
    url = ExchangeMonitor + "/ajax/widget-unique"
    all_monitors = {}
    all_monitors = {'datetime': get_time_zone()}
    
    for type in types:
        params = {
            "country": country,
            "type": type
        }
        response = ask(url, params=params)
        dataJson = json.loads(response)

        price = str(dataJson['price']).replace(',', '.')
        price = price.replace('.', '', 1) if price.count('.') == 2 else price
        change = f"{dataJson['percent']}" if dataJson['symbol'] == 'neutral' else f"{dataJson['percent']}"
        data = {
            "title": dataJson['name'],
            "price": price,
            "change": change
        }

        all_monitors[_convert_specific_format(dataJson['name'])] = data
        
    return all_monitors


import json

from datetime import datetime, timedelta
from babel.dates import format_date, format_time
from pyDolarVenezuela.request import content
from pyDolarVenezuela.util import WorldTime

response = content(WorldTime)
datetime_obj = json.loads(response)['datetime']

def get_time(date: str) -> (datetime | None):
    datetime_f = datetime.fromisoformat(datetime_obj)
    listdate = date.split(' ')

    if len(listdate) == 3:
        time = listdate[-1]
        
        if listdate[1] == "un" or listdate[1] == "una":
            hms = 1
        else:
            hms = int(listdate[1])

        time_units = {
            "semana": "weeks",
            "semanas": "weeks",
            "día": "days", "días": "days",
            "horas": "hours", "hora": "hours",
            "minutos": "minutes", "minuto": "minutes",
            "segundos": "seconds"
        }
        
        duration = timedelta(**{time_units[time]: hms})
        exact_time = datetime_f - duration

        if time == "día" or time == "días":
            return exact_time.strftime("%d/%m/%Y") 
         
        return exact_time.strftime("%d/%m/%Y, %I:%M %p")
    
    elif len(listdate) == 5:
        duration = timedelta(days=30)

        exact_time = datetime_f - duration
        return exact_time.strftime("%d/%m/%Y")
    
    else:
        return None 

def get_time_zone():
    datetime_str = datetime.strptime(datetime_obj, "%Y-%m-%dT%H:%M:%S.%f%z")

    formatted_date = format_date(datetime_str.date(), format='full', locale='es')
    formatted_time = format_time(datetime_str.time(), format='h:mm:ss a', locale='es')

    return {
        "date": formatted_date,
        "time": formatted_time
    }


from pyBCV import Currency
from decimal import Decimal

class Bcv:
    def get_rates(self, currency_code: str, prettify: bool = True):
        round_number_rate = Decimal(Currency().get_rate(currency_code=currency_code, prettify=False))
        round_number_rate = round(round_number_rate, 2)
        return round_number_rate if not prettify else f"Bs. {round_number_rate:,.2f}"



from bs4 import BeautifulSoup

from functionstime import get_time, get_time_zone
from request import content
from util import ExchangeMonitor

def _get_values_monitors(soup: BeautifulSoup):
    return [value for value in soup]

def _convert_specific_format(text: str, character: str = '_') -> str:
    acentos = {'á': 'a', 'é': 'e', 'í': 'i', 'ó': 'o', 'ú': 'u'}
    for acento, sin_acento in acentos.items():
        text = text.lower().replace(acento, sin_acento).replace(' ', character)
    return text

class Monitor(object):
    def __init__(self) -> None:
        """
        La clase Monitor permite consultar los precios del dólar en diversos monitores en Venezuela. \n
        El método `_screaped` carga los datos de los monitores a través del scraping de la página web de referencia, donde los datos son almacenados en un diccionario. \ 
        El método `get_value_monitors` permite acceder a los datos almacenados en el diccionario.
        """
        self.url = ExchangeMonitor + "/dolar-venezuela"
    
    def _scraped(self):
        response = content(self.url)
        soup = BeautifulSoup(response, "html.parser")

        section_dolar_venezuela = soup.find_all("div", "col-xs-12 col-sm-6 col-md-4 col-tabla")
        _scraping_monitors = _get_values_monitors(section_dolar_venezuela)
        
        self.data = {}
        self.data['datetime'] = get_time_zone()

        for scraping_monitor in _scraping_monitors:
            result = scraping_monitor.find("div", "module-table module-table-fecha")

            name  = result.find("h6", "nombre").text
            price = str(result.find('p', "precio").text).replace(',', '.')

            if price.count('.') == 2:
                price = price.replace('.', '', 1)
            
            last_update = get_time(' '.join(str(result.find('p', "fecha").text).split(' ')[1:]).capitalize())
            symbol = str(result.find('p', "cambio-por").text)[0]
            color  = "red" if symbol == '▼' else "green" if symbol == '▲' else "neutral"
            percent = str(result.find('p', "cambio-por").text)[1:].strip()
            change = str(result.find('p', "cambio-num").text)

            data = {
                "title": name,
                "price": price,
                "last_update": last_update,
                "percent": percent,
                "change": change,
                "color": color
            }

            self.data[_convert_specific_format(name)] = data

    def get_value_monitors(self, monitor_code: str = None, name_property: str = None, prettify: bool = False):
        """
        El parámetro `monitor_code` indica el código del monitor del cual se desea obtener información, \
        mientras que el parámetro `prettify` permite mostrar los precios en formato de moneda con el símbolo de Bolívares. \
        Si se proporciona un nombre de propiedad válido, se devolverá el valor correspondiente para ese monitor.
        """
        self._scraped()
        monitor_code = monitor_code.lower()

        if not monitor_code:
            return self.data
        if monitor_code not in self.data:
            raise ValueError("Does not match any of the properties that were provided in the dictionary. Most information: https://github.com/fcoagz/pyDolarVenezuela")
        
        return (f"Bs. {self.data[monitor_code][name_property]}" if prettify and name_property == "price"
                else self.data[monitor_code][name_property] if name_property in self.data[monitor_code]
                else self.data[monitor_code])


from httpx import Response
import httpx

def _ensure_200_and_return_content(response: Response) -> bytes:
    if response.status_code != httpx.codes.OK:
        raise ValueError("")
    return response.content

def ask(url: str, params: dict):
    response = httpx.get(url, params=params, timeout=10.0)
    return _ensure_200_and_return_content(response)

def content(url: str) -> bytes:
    response = httpx.get(url, timeout=10.0)
    return _ensure_200_and_return_content(response)


import pathlib
from setuptools import find_packages, setup

HERE = pathlib.Path(__file__).parent

VERSION = '1.1.8'
PACKAGE_NAME = 'pyDolarVenezuela' 
AUTHOR = 'Francisco Griman'
AUTHOR_EMAIL = 'grihardware@gmail.com'
URL = 'https://github.com/fcoagz/pydolarvenezuela'

LICENSE = 'MIT'
DESCRIPTION = 'esta es una librería en python que te permite consultar los precios del dólar en diferentes monitores en Venezuela y el dolar oficial BCV.'
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding='utf-8')
LONG_DESC_TYPE = "text/markdown"

INSTALL_REQUIRES = [
      'requests',
      'bs4',
      'beautifulsoup4',
      'pyBCV',
      'babel',
      'httpx'
      ]

CLASSIFIERS = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    install_requires=INSTALL_REQUIRES,
    license=LICENSE,
    packages=find_packages(),
    include_package_data=True,
    classifiers=CLASSIFIERS
)